import os

from setuptools import Extension, Distribution
import numpy
from Cython.Build import cythonize
from Cython.Distutils.build_ext import new_build_ext as cython_build_ext


def build():
    # compile FLI library
    os.system('cd lib && make && cd ..')

    extensions = [
        Extension(
            'pyobs_fli.flidriver',
            ['pyobs_fli/flidriver.pyx'],
            library_dirs=['lib/'],
            libraries=['fli', 'cfitsio'],
            include_dirs=[numpy.get_include()],
            extra_compile_args=['-fPIC']
        )
    ]
    ext_modules = cythonize(extensions)

    distribution = Distribution({
        "name": "extended",
        "ext_modules": ext_modules,
        "cmdclass": {
            "build_ext": cython_build_ext,
        },
    })

    distribution.run_command("build_ext")
    build_ext_cmd = distribution.get_command_obj("build_ext")
    build_ext_cmd.copy_extensions_to_source()


if __name__ == "__main__":
    build()



# distutils: language = c++

from collections import namedtuple
from enum import Enum
from typing import Tuple, List

import numpy as np
cimport numpy as np
np.import_array()

from libfli cimport *


DeviceInfo = namedtuple('DeviceInfo', ['domain', 'filename', 'name'])


class FliTemperature(Enum):
    """Enumeration for temperature sensors."""
    INTERNAL = FLI_TEMPERATURE_INTERNAL
    EXTERNAL = FLI_TEMPERATURE_EXTERNAL
    CCD =  FLI_TEMPERATURE_CCD
    BASE = FLI_TEMPERATURE_BASE


cdef class FliDriver:
    """Wrapper for the FLI driver."""

    @staticmethod
    def list_devices() -> List[DeviceInfo]:
        """List all FLI USB cameras connected to this computer.

        Returns:
            List of DeviceInfo tuples.
        """

        # define variables
        cdef flidomain_t domain
        cdef char filename[1024]
        cdef char name[1024]

        # create list of USB camera
        if FLICreateList(FLIDOMAIN_USB | FLIDEVICE_CAMERA) != 0:
            raise ValueError('Could not create list of FLI cameras.')

        # init list of devices
        devices = []

        # get first camera
        if FLIListFirst(&domain, <char*>filename, 1024, <char*>name, 1024) == 0:
            # store first device
            devices.append(DeviceInfo(domain=domain, filename=filename, name=name))

            # loop other devices
            while FLIListNext(&domain, <char*>filename, 1024, <char*>name, 1024) == 0:
                # store device
                devices.append(DeviceInfo(domain=domain, filename=filename, name=name))

        # clean up and return
        FLIDeleteList()
        return devices

    """Storage for the device info."""
    cdef object _device_info

    """Storage for link to device."""
    cdef flidev_t _device

    def __init__(self, device_info: DeviceInfo):
        """Create a new driver object for the given device.

        Args:
            device_info: A DeviceInfo obtained from list_devices.
        """
        self._device_info = device_info

    def open(self) -> None:
        """Open driver.

        Raises:
            ValueError: If opening failed.
        """
        res = FLIOpen(&self._device, self._device_info.filename, self._device_info.domain)
        if res != 0:
            raise ValueError('Could not open device.')

    def close(self) -> None:
        """Close driver.

        Raises:
            ValueError: If closing failed.
        """
        res = FLIClose(self._device)
        if res != 0:
            raise ValueError('Could not open device.')

    @property
    def name(self) -> str:
        """Returns the name of the connected device."""
        return self._device_info.name.decode('utf-8')

    def get_window_binning(self) -> Tuple[Tuple[int, int, int, int], Tuple[int, int]]:
        """Get tuple of window and binning dicts.

        Returns:
            Two tuples with (left, top, width, height) and (xbin, ybin)

        Raises:
            ValueError: If fetching readout dimensions failed.
        """

        # variables
        cdef long width, hoffset, hbin, height, voffset, vbin
        
        # get dimensions
        res = FLIGetReadoutDimensions(self._device, &width, &hoffset, &hbin, &height, &voffset, &vbin)
        if res != 0:
            raise ValueError('Could not query readout dimensions.')

        # return window and binning
        return (hoffset, voffset, width, height), (hbin, vbin)

    def get_visible_frame(self) -> Tuple[int, int, int, int]:
        """Returns the visible frame of the connected camera.

        Returns:
            Tuple with left, top, width, and height of full frame.

        Raises:
            ValueError: If fetching visible area fails.
        """

        # variables
        cdef long ul_x, ul_y, lr_x, lr_y

        # get area
        res = FLIGetVisibleArea(self._device, &ul_x, &ul_y, &lr_x, &lr_y)
        if res != 0:
            raise ValueError('Could not query visible area.')

        # return it
        return ul_x, ul_y, lr_x -  ul_x, lr_y - ul_y

    def get_full_frame(self) -> Tuple[int, int, int, int]:
        """Returns the full frame of the connected camera.

        Returns:
            Tuple with left, top, width, and height of full frame.

        Raises:
            ValueError: If fetching visible area fails.
        """

        # variables
        cdef long ul_x, ul_y, lr_x, lr_y

        # get area
        res = FLIGetArrayArea(self._device, &ul_x, &ul_y, &lr_x, &lr_y)
        if res != 0:
            raise ValueError('Could not query total area.')

        # return it
        return ul_x, ul_y, lr_x -  ul_x, lr_y - ul_y

    def set_binning(self, x: int, y: int) -> None:
        """Set the binning.

        Args:
            x: Binning in x direction.
            y: Binning in y direction.

        Raises:
            ValueError: If setting binning failed.
        """

        # set x binning
        res = FLISetHBin(self._device, x)
        if res != 0:
            raise ValueError('Could not set x binning.')

        # set y binning
        res = FLISetVBin(self._device, y)
        if res != 0:
            raise ValueError('Could not set y binning.')

    def set_window(self, left: int, top: int, width: int, height: int) -> None:
        """Sets the window.

        Args:
            left: X offset of window.
            top: Y offset of window.
            width: Window width.
            height: Window height.

        Raises:
            ValueError: If setting the window failed.
        """

        # set window
        res = FLISetImageArea(self._device, left, top, left + width, top + height)
        if res != 0:
            raise ValueError('Could not set window.')

    def init_exposure(self, open_shutter: bool) -> None:
        """Initializes an exposure.

        Args:
            open_shutter: Whether the shutter should be opened for exposure.

        Raises:
            ValueError: If initialization failed.
        """

        # set TDI
        res = FLISetTDI(self._device, 0, 0)
        if res != 0:
            raise ValueError('Could not set TDI.')

        # set frame type
        res = FLISetFrameType(self._device, FLI_FRAME_TYPE_NORMAL if open_shutter else FLI_FRAME_TYPE_DARK)
        if res != 0:
            raise ValueError('Could not set frame type.')

    def set_exposure_time(self, exptime: int) -> None:
        """Sets the exposure time.

        Args:
            exptime: Exposure time in ms.

        Raises:
            ValueError: If setting of exposure time failed.
        """

        # set exptime
        res = FLISetExposureTime(self._device, exptime)
        if res != 0:
            raise ValueError('Could not set exposure time.')

    def start_exposure(self) -> None:
        """Start a new exposure.

        Raises:
            ValueError: If starting the exposure failed.
        """

        # expose
        res = FLIExposeFrame(self._device)
        if res != 0:
            raise ValueError('Could not start exposure.')

    def is_exposing(self) -> bool:
        """Checks, whether the camera is currently exposing

        Returns:
            bool: Whether camera is currently exposing.

        Raises:
            ValueError: If fetching device or exposure status failed.
        """

        # variables
        cdef long status, timeleft

        # get status
        res = FLIGetDeviceStatus(self._device, &status)
        if res != 0:
            raise ValueError('Could not fetch device status.')
        res = FLIGetExposureStatus(self._device, &timeleft)
        if res != 0:
            raise ValueError('Could not fetch remaining exposure time.')

        # finished?
        return (status == FLI_CAMERA_STATUS_UNKNOWN and timeleft == 0) or \
               (status != FLI_CAMERA_STATUS_UNKNOWN and status & FLI_CAMERA_DATA_READY)

    def get_temp(self, channel: FliTemperature) -> float:
        """Returns the temperature of the given sensor.

        Args:
            channel: Sensor to read out.

        Returns:
            float: Temperature of given sensor.

        Raises:
            ValueError: If fetching temperature failed.
        """

        # variables
        cdef double temp

        # get it
        res = FLIReadTemperature(self._device, channel.value, &temp)
        if res != 0:
            raise ValueError('Could not fetch temperature.')

        # return it
        return temp

    def get_cooler_power(self) -> float:
        """Get power of cooling in percent.

        Returns:
            float: Current cooling power.

        Raises:
            ValueError: If fetching cooling power failed.
        """

        # variables
        cdef double power

        # get it
        res = FLIGetCoolerPower(self._device, &power)
        if res != 0:
            raise ValueError('Could not fetch cooler power.')

        # return it
        return power

    def grab_row(self, width: int) -> np.ndarray:
        """Reads out a row from the camera.

        Args:
            width: Width of row to read out.

        Returns:
            ndarray: Data of row.

        Raises:
            ValueError: If reading row failed.
        """

        # create numpy array of given dimensions
        cdef np.ndarray[unsigned short, ndim=1] row = np.zeros((width,), dtype=np.ushort)

        # get pointer to data
        cdef void* row_data = <void*> row.data

        # call library
        res = FLIGrabRow(self._device, row_data, width)
        if res != 0:
            raise ValueError('Could not grab row from camera.')

        # return row
        return row

    def cancel_exposure(self) -> None:
        """Cancel an exposure.

        Raises:
            ValueError: If canceling failed.

        """
        res = FLICancelExposure(self._device)
        if res != 0:
            raise ValueError('Could not cancel exposure.')

    def set_temperature(self, setpoint: float) -> None:
        """Set cooling emperature setpoint.

        Args:
            setpoint: New temperature setpoing.

        Raises:
            ValueError: If setting temperature failed.
        """
        res = FLISetTemperature(self._device, setpoint)
        if res != 0:
            raise ValueError('Could not set temperature.')



import asyncio
import logging
import math
from datetime import datetime
import time
from typing import Tuple, Any, Optional, Dict
import numpy as np

from pyobs.interfaces import ICamera, IWindow, IBinning, ICooling
from pyobs.modules.camera.basecamera import BaseCamera
from pyobs.images import Image
from pyobs.utils.enums import ExposureStatus
from pyobs.utils import exceptions as exc

from .flidriver import FliDriver, FliTemperature


log = logging.getLogger(__name__)


class FliCamera(BaseCamera, ICamera, IWindow, IBinning, ICooling):
    """A pyobs module for FLI cameras."""

    __module__ = "pyobs_fli"

    def __init__(self, setpoint: float = -20.0, **kwargs: Any):
        """Initializes a new FliCamera.

        Args:
            setpoint: Cooling temperature setpoint.
        """
        BaseCamera.__init__(self, **kwargs)

        # variables
        self._driver: Optional[FliDriver] = None
        self._temp_setpoint: Optional[float] = setpoint

        # window and binning
        self._window = (0, 0, 0, 0)
        self._binning = (1, 1)

    async def open(self) -> None:
        """Open module."""
        await BaseCamera.open(self)

        # list devices
        devices = FliDriver.list_devices()
        if len(devices) == 0:
            raise ValueError("No camera found.")

        # open first one
        d = devices[0]
        log.info('Opening connection to "%s" at %s...', d.name.decode("utf-8"), d.filename.decode("utf-8"))
        self._driver = FliDriver(d)
        try:
            self._driver.open()
        except ValueError as e:
            raise ValueError("Could not open FLI camera: %s", e)

        # get window and binning from camera
        self._window, self._binning = self._driver.get_window_binning()

        # set cooling
        if self._temp_setpoint is not None:
            await self.set_cooling(True, self._temp_setpoint)

    async def close(self) -> None:
        """Close the module."""
        await BaseCamera.close(self)

        # not open?
        if self._driver is not None:
            # close connection
            self._driver.close()
            self._driver = None

    async def get_full_frame(self, **kwargs: Any) -> Tuple[int, int, int, int]:
        """Returns full size of CCD.

        Returns:
            Tuple with left, top, width, and height set.
        """
        if self._driver is None:
            raise ValueError("No camera driver.")
        return self._driver.get_full_frame()

    async def get_window(self, **kwargs: Any) -> Tuple[int, int, int, int]:
        """Returns the camera window.

        Returns:
            Tuple with left, top, width, and height set.
        """
        return self._window

    async def get_binning(self, **kwargs: Any) -> Tuple[int, int]:
        """Returns the camera binning.

        Returns:
            Tuple with x and y.
        """
        return self._binning

    async def set_window(self, left: int, top: int, width: int, height: int, **kwargs: Any) -> None:
        """Set the camera window.

        Args:
            left: X offset of window.
            top: Y offset of window.
            width: Width of window.
            height: Height of window.

        Raises:
            ValueError: If binning could not be set.
        """
        self._window = (left, top, width, height)
        log.info("Setting window to %dx%d at %d,%d...", width, height, left, top)

    async def set_binning(self, x: int, y: int, **kwargs: Any) -> None:
        """Set the camera binning.

        Args:
            x: X binning.
            y: Y binning.

        Raises:
            ValueError: If binning could not be set.
        """
        self._binning = (x, y)
        log.info("Setting binning to %dx%d...", x, y)

    async def _expose(self, exposure_time: float, open_shutter: bool, abort_event: asyncio.Event) -> Image:
        """Actually do the exposure, should be implemented by derived classes.

        Args:
            exposure_time: The requested exposure time in seconds.
            open_shutter: Whether or not to open the shutter.
            abort_event: Event that gets triggered when exposure should be aborted.

        Returns:
            The actual image.

        Raises:
            GrabImageError: If exposure was not successful.
        """

        # check driver
        if self._driver is None:
            raise ValueError("No camera driver.")

        # set binning
        log.info("Set binning to %dx%d.", self._binning[0], self._binning[1])
        self._driver.set_binning(*self._binning)

        # set window, divide width/height by binning, from libfli:
        # "Note that the given lower-right coordinate must take into account the horizontal and
        # vertical bin factor settings, but the upper-left coordinate is absolute."
        width = int(math.floor(self._window[2]) / self._binning[0])
        height = int(math.floor(self._window[3]) / self._binning[1])
        log.info(
            "Set window to %dx%d (binned %dx%d) at %d,%d.",
            self._window[2],
            self._window[3],
            width,
            height,
            self._window[0],
            self._window[1],
        )
        self._driver.set_window(self._window[0], self._window[1], width, height)

        # set some stuff
        self._driver.init_exposure(open_shutter)
        self._driver.set_exposure_time(int(exposure_time * 1000.0))

        # get date obs
        log.info(
            "Starting exposure with %s shutter for %.2f seconds...", "open" if open_shutter else "closed", exposure_time
        )
        date_obs = datetime.utcnow().strftime("%Y-%m-%dT%H:%M:%S.%f")

        # do exposure
        self._driver.start_exposure()

        # wait for exposure to finish
        while True:
            # aborted?
            if abort_event.is_set():
                await self._change_exposure_status(ExposureStatus.IDLE)
                raise InterruptedError("Aborted exposure.")

            # is exposure finished?
            if self._driver.is_exposing():
                break
            else:
                # sleep a little
                time.sleep(0.2)

        # readout
        log.info("Exposure finished, reading out...")
        await self._change_exposure_status(ExposureStatus.READOUT)
        width = int(math.floor(self._window[2] / self._binning[0]))
        height = int(math.floor(self._window[3] / self._binning[1]))
        img = np.zeros((height, width), dtype=np.uint16)
        for row in range(height):
            img[row, :] = self._driver.grab_row(width)

        # create FITS image and set header
        image = Image(img)
        image.header["DATE-OBS"] = (date_obs, "Date and time of start of exposure")
        image.header["EXPTIME"] = (exposure_time, "Exposure time [s]")
        image.header["DET-TEMP"] = (self._driver.get_temp(FliTemperature.CCD), "CCD temperature [C]")
        image.header["DET-COOL"] = (self._driver.get_cooler_power(), "Cooler power [percent]")
        image.header["DET-TSET"] = (self._temp_setpoint, "Cooler setpoint [C]")

        # instrument and detector
        image.header["INSTRUME"] = (self._driver.name, "Name of instrument")

        # binning
        image.header["XBINNING"] = image.header["DET-BIN1"] = (self._binning[0], "Binning factor used on X axis")
        image.header["YBINNING"] = image.header["DET-BIN2"] = (self._binning[1], "Binning factor used on Y axis")

        # window
        image.header["XORGSUBF"] = (self._window[0], "Subframe origin on X axis")
        image.header["YORGSUBF"] = (self._window[1], "Subframe origin on Y axis")

        # statistics
        image.header["DATAMIN"] = (float(np.min(img)), "Minimum data value")
        image.header["DATAMAX"] = (float(np.max(img)), "Maximum data value")
        image.header["DATAMEAN"] = (float(np.mean(img)), "Mean data value")

        # biassec/trimsec
        full = self._driver.get_visible_frame()
        self.set_biassec_trimsec(image.header, *full)

        # return FITS image
        log.info("Readout finished.")
        return image

    async def _abort_exposure(self) -> None:
        """Abort the running exposure. Should be implemented by derived class.

        Raises:
            ValueError: If an error occured.
        """
        if self._driver is None:
            raise ValueError("No camera driver.")
        self._driver.cancel_exposure()

    async def get_cooling_status(self, **kwargs: Any) -> Tuple[bool, float, float]:
        """Returns the current status for the cooling.

        Returns:
            Tuple containing:
                Enabled (bool):         Whether the cooling is enabled
                SetPoint (float):       Setpoint for the cooling in celsius.
                Power (float):          Current cooling power in percent or None.
        """
        if self._driver is None:
            raise ValueError("No camera driver.")
        enabled = self._temp_setpoint is not None
        return (
            enabled,
            self._temp_setpoint if self._temp_setpoint is not None else 99.0,
            self._driver.get_cooler_power(),
        )

    async def get_temperatures(self, **kwargs: Any) -> Dict[str, float]:
        """Returns all temperatures measured by this module.

        Returns:
            Dict containing temperatures.
        """
        if self._driver is None:
            raise ValueError("No camera driver.")
        return {"CCD": self._driver.get_temp(FliTemperature.CCD), "Base": self._driver.get_temp(FliTemperature.BASE)}

    async def set_cooling(self, enabled: bool, setpoint: float, **kwargs: Any) -> None:
        """Enables/disables cooling and sets setpoint.

        Args:
            enabled: Enable or disable cooling.
            setpoint: Setpoint in celsius for the cooling.

        Raises:
            ValueError: If cooling could not be set.
        """
        if self._driver is None:
            raise ValueError("No camera driver.")

        # log
        if enabled:
            log.info("Enabling cooling with a setpoint of %.2f°C...", setpoint)
        else:
            log.info("Disabling cooling and setting setpoint to 20°C...")

        # if not enabled, set setpoint to None
        self._temp_setpoint = setpoint if enabled else None

        # set setpoint
        self._driver.set_temperature(float(setpoint) if setpoint is not None else 20.0)


__all__ = ["FliCamera"]



cdef extern from "../lib/libfli.h":
    # An opaque handle used by library functions to refer to FLI
    # hardware.
    cdef int FLI_INVALID_DEVICE
    ctypedef long flidev_t

    # The domain of an FLI device.  This consists of a bitwise ORed
    # combination of interface method and device type.  Valid interfaces
    # are \texttt{FLIDOMAIN_PARALLEL_PORT}, \texttt{FLIDOMAIN_USB},
    # \texttt{FLIDOMAIN_SERIAL}, and \texttt{FLIDOMAIN_INET}.  Valid
    # device types are \texttt{FLIDEVICE_CAMERA},
    # \texttt{FLIDOMAIN_FILTERWHEEL}, and \texttt{FLIDOMAIN_FOCUSER}.
    #
    # @see FLIOpen
    # @see FLIList
    ctypedef long flidomain_t

    cdef int FLIDOMAIN_NONE
    cdef int FLIDOMAIN_PARALLEL_PORT
    cdef int FLIDOMAIN_USB
    cdef int FLIDOMAIN_SERIAL
    cdef int FLIDOMAIN_INET
    cdef int FLIDOMAIN_SERIAL_19200
    cdef int FLIDOMAIN_SERIAL_1200

    cdef int FLIDEVICE_NONE
    cdef int FLIDEVICE_CAMERA
    cdef int FLIDEVICE_FILTERWHEEL
    cdef int FLIDEVICE_FOCUSER
    cdef int FLIDEVICE_HS_FILTERWHEEL
    cdef int FLIDEVICE_RAW
    cdef int FLIDEVICE_ENUMERATE_BY_CONNECTION

    # The frame type for an FLI CCD camera device.  Valid frame types are
    # \texttt{FLI_FRAME_TYPE_NORMAL} and \texttt{FLI_FRAME_TYPE_DARK}.
    #
    # @see FLISetFrameType
    ctypedef long fliframe_t

    cdef int FLI_FRAME_TYPE_NORMAL
    cdef int FLI_FRAME_TYPE_DARK
    cdef int FLI_FRAME_TYPE_FLOOD
    cdef int FLI_FRAME_TYPE_RBI_FLUSH

    # The gray-scale bit depth for an FLI camera device.  Valid bit
    # depths are \texttt{FLI_MODE_8BIT} and \texttt{FLI_MODE_16BIT}.
    #
    # @see FLISetBitDepth
    ctypedef long flibitdepth_t

    cdef int FLI_MODE_8BIT
    cdef int FLI_MODE_16BIT

    # Type used for shutter operations for an FLI camera device.  Valid
    # shutter types are \texttt{FLI_SHUTTER_CLOSE},
    # \texttt{FLI_SHUTTER_OPEN},
    # \texttt{FLI_SHUTTER_EXTERNAL_TRIGGER},
    # \texttt{FLI_SHUTTER_EXTERNAL_TRIGGER_LOW}, and
    # \texttt{FLI_SHUTTER_EXTERNAL_TRIGGER_HIGH}.
    #
    # @see FLIControlShutter
    ctypedef long flishutter_t

    cdef int FLI_SHUTTER_CLOSE
    cdef int FLI_SHUTTER_OPEN
    cdef int FLI_SHUTTER_EXTERNAL_TRIGGER
    cdef int FLI_SHUTTER_EXTERNAL_TRIGGER_LOW
    cdef int FLI_SHUTTER_EXTERNAL_TRIGGER_HIGH
    cdef int FLI_SHUTTER_EXTERNAL_EXPOSURE_CONTROL

    # Type used for background flush operations for an FLI camera device.  Valid
    # bgflush types are \texttt{FLI_BGFLUSH_STOP} and
    # \texttt{FLI_BGFLUSH_START}.
    #
    # @see FLIControlBackgroundFlush
    ctypedef long flibgflush_t

    cdef int FLI_BGFLUSH_STOP
    cdef int FLI_BGFLUSH_START

    # Type used to determine which temperature channel to read.  Valid
    # channel types are \texttt{FLI_TEMPERATURE_INTERNAL} and
    # \texttt{FLI_TEMPERATURE_EXTERNAL}.
    #
    # @see FLIReadTemperature
    ctypedef long flichannel_t

    cdef int FLI_TEMPERATURE_INTERNAL
    cdef int FLI_TEMPERATURE_EXTERNAL
    cdef int FLI_TEMPERATURE_CCD
    cdef int FLI_TEMPERATURE_BASE

    # Type specifying library debug levels.  Valid debug levels are
    # \texttt{FLIDEBUG_NONE}, \texttt{FLIDEBUG_INFO},
    # \texttt{FLIDEBUG_WARN}, and \texttt{FLIDEBUG_FAIL}.
    #
    # @see FLISetDebugLevel
    ctypedef long flidebug_t
    ctypedef long flimode_t
    ctypedef long flistatus_t
    ctypedef long flitdirate_t
    ctypedef long flitdiflags_t

    # Status settings
    cdef int FLI_CAMERA_STATUS_UNKNOWN
    cdef int FLI_CAMERA_STATUS_MASK
    cdef int FLI_CAMERA_STATUS_IDLE
    cdef int FLI_CAMERA_STATUS_WAITING_FOR_TRIGGER
    cdef int FLI_CAMERA_STATUS_EXPOSING
    cdef int FLI_CAMERA_STATUS_READING_CCD
    cdef int FLI_CAMERA_DATA_READY

    cdef int FLI_FOCUSER_STATUS_UNKNOWN
    cdef int FLI_FOCUSER_STATUS_HOMING
    cdef int FLI_FOCUSER_STATUS_MOVING_IN
    cdef int FLI_FOCUSER_STATUS_MOVING_OUT
    cdef int FLI_FOCUSER_STATUS_MOVING_MASK
    cdef int FLI_FOCUSER_STATUS_HOME
    cdef int FLI_FOCUSER_STATUS_LIMIT
    cdef int FLI_FOCUSER_STATUS_LEGACY

    cdef int FLI_FILTER_WHEEL_PHYSICAL
    cdef int FLI_FILTER_WHEEL_VIRTUAL
    cdef int FLI_FILTER_WHEEL_LEFT
    cdef int FLI_FILTER_WHEEL_RIGHT
    cdef int FLI_FILTER_STATUS_MOVING_CCW
    cdef int FLI_FILTER_STATUS_MOVING_CW
    cdef int FLI_FILTER_POSITION_UNKNOWN
    cdef int FLI_FILTER_POSITION_CURRENT
    cdef int FLI_FILTER_STATUS_HOMING
    cdef int FLI_FILTER_STATUS_HOME
    cdef int FLI_FILTER_STATUS_HOME_LEFT
    cdef int FLI_FILTER_STATUS_HOME_RIGHT
    cdef int FLI_FILTER_STATUS_HOME_SUCCEEDED

    cdef int FLIDEBUG_NONE
    cdef int FLIDEBUG_INFO
    cdef int FLIDEBUG_WARN
    cdef int FLIDEBUG_FAIL
    cdef int FLIDEBUG_IO
    cdef int FLIDEBUG_ALL

    cdef int FLI_IO_P0
    cdef int FLI_IO_P1
    cdef int FLI_IO_P2
    cdef int FLI_IO_P3

    cdef int FLI_FAN_SPEED_OFF
    cdef int FLI_FAN_SPEED_ON

    cdef int FLI_EEPROM_USER
    cdef int FLI_EEPROM_PIXEL_MAP

    cdef int FLI_PIXEL_DEFECT_COLUMN
    cdef int FLI_PIXEL_DEFECT_CLUSTER
    cdef int FLI_PIXEL_DEFECT_POINT_BRIGHT
    cdef int FLI_PIXEL_DEFECT_POINT_DARK
    
    long FLIOpen(flidev_t *dev, char *name, flidomain_t domain)
    long FLISetDebugLevel(char *host, flidebug_t level)
    long FLIClose(flidev_t dev)
    long FLIGetLibVersion(char* ver, size_t len)
    long FLIGetModel(flidev_t dev, char* model, size_t len)
    long FLIGetPixelSize(flidev_t dev, double *pixel_x, double *pixel_y)
    long FLIGetHWRevision(flidev_t dev, long *hwrev)
    long FLIGetFWRevision(flidev_t dev, long *fwrev)
    long FLIGetArrayArea(flidev_t dev, long* ul_x, long* ul_y, long* lr_x, long* lr_y)
    long FLIGetVisibleArea(flidev_t dev, long* ul_x, long* ul_y, long* lr_x, long* lr_y)
    long FLISetExposureTime(flidev_t dev, long exptime)
    long FLISetImageArea(flidev_t dev, long ul_x, long ul_y, long lr_x, long lr_y)
    long FLISetHBin(flidev_t dev, long hbin)
    long FLISetVBin(flidev_t dev, long vbin)
    long FLISetFrameType(flidev_t dev, fliframe_t frametype)
    long FLICancelExposure(flidev_t dev)
    long FLIGetExposureStatus(flidev_t dev, long *timeleft)
    long FLISetTemperature(flidev_t dev, double temperature)
    long FLIGetTemperature(flidev_t dev, double *temperature)
    long FLIGetCoolerPower(flidev_t dev, double *power)
    long FLIGrabRow(flidev_t dev, void *buff, size_t width)
    long FLIExposeFrame(flidev_t dev)
    long FLIFlushRow(flidev_t dev, long rows, long repeat)
    long FLISetNFlushes(flidev_t dev, long nflushes)
    long FLISetBitDepth(flidev_t dev, flibitdepth_t bitdepth)
    long FLIReadIOPort(flidev_t dev, long *ioportset)
    long FLIWriteIOPort(flidev_t dev, long ioportset)
    long FLIConfigureIOPort(flidev_t dev, long ioportset)
    long FLILockDevice(flidev_t dev)
    long FLIUnlockDevice(flidev_t dev)
    long FLIControlShutter(flidev_t dev, flishutter_t shutter)
    long FLIControlBackgroundFlush(flidev_t dev, flibgflush_t bgflush)
    long FLISetDAC(flidev_t dev, unsigned long dacset)
    long FLIList(flidomain_t domain, char ***names)
    long FLIFreeList(char **names)
    
    long FLIGetFilterName(flidev_t dev, long filter, char *name, size_t len)
    long FLISetActiveWheel(flidev_t dev, long wheel)
    long FLIGetActiveWheel(flidev_t dev, long *wheel)
    
    long FLISetFilterPos(flidev_t dev, long filter)
    long FLIGetFilterPos(flidev_t dev, long *filter)
    long FLIGetFilterCount(flidev_t dev, long *filter)
    
    long FLIStepMotor(flidev_t dev, long steps)
    long FLIStepMotorAsync(flidev_t dev, long steps)
    long FLIGetStepperPosition(flidev_t dev, long *position)
    long FLIGetStepsRemaining(flidev_t dev, long *steps)
    long FLIHomeFocuser(flidev_t dev)
    long FLICreateList(flidomain_t domain)
    long FLIDeleteList()
    long FLIListFirst(flidomain_t *domain, char *filename, size_t fnlen, char *name, size_t namelen)
    long FLIListNext(flidomain_t *domain, char *filename, size_t fnlen, char *name, size_t namelen)
    long FLIReadTemperature(flidev_t dev, flichannel_t channel, double *temperature)
    long FLIGetFocuserExtent(flidev_t dev, long *extent)
    long FLIUsbBulkIO(flidev_t dev, int ep, void *buf, long *len)
    long FLIGetDeviceStatus(flidev_t dev, long *status)
    long FLIGetCameraModeString(flidev_t dev, flimode_t mode_index, char *mode_string, size_t siz)
    long FLIGetCameraMode(flidev_t dev, flimode_t *mode_index)
    long FLISetCameraMode(flidev_t dev, flimode_t mode_index)
    long FLIHomeDevice(flidev_t dev)
    long FLIGrabFrame(flidev_t dev, void* buff, size_t buffsize, size_t* bytesgrabbed)
    long FLISetTDI(flidev_t dev, flitdirate_t tdi_rate, flitdiflags_t flags)
    long FLIGrabVideoFrame(flidev_t dev, void *buff, size_t size)
    long FLIStopVideoMode(flidev_t dev)
    long FLIStartVideoMode(flidev_t dev)
    long FLIGetSerialString(flidev_t dev, char* serial, size_t len)
    long FLIEndExposure(flidev_t dev)
    long FLITriggerExposure(flidev_t dev)
    long FLISetFanSpeed(flidev_t dev, long fan_speed)
    long FLISetVerticalTableEntry(flidev_t dev, long index, long height, long bin, long mode)
    long FLIGetVerticalTableEntry(flidev_t dev, long index, long *height, long *bin, long *mode)
    long FLIGetReadoutDimensions(flidev_t dev, long *width, long *hoffset, long *hbin, long *height, long *voffset, long *vbin)
    long FLIEnableVerticalTable(flidev_t dev, long width, long offset, long flags)
    long FLIReadUserEEPROM(flidev_t dev, long loc, long address, long length, void *rbuf)
    long FLIWriteUserEEPROM(flidev_t dev, long loc, long address, long length, void *wbuf)
